#!/usr/bin/env python

"""
A ROM upload utility for the Arduino-Am29F010 interface.

Copyright (C) 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from serial import Serial
from time import sleep
import sys

port = "/dev/ttyUSB0"
rate = 115200

def openPort():
    global s
    s = Serial(port, rate)
    s.setTimeout(5)
    s.readline()

def closePort():
    global s
    s.close()
    s = None

def writeToPort(data):
    s.write(data)
    s.flush()
    print >>sys.stderr, data.rstrip()

def readFromPort():
    resp = s.readline().rstrip("\r\n")
    if resp == "?":
        resp = s.readline().rstrip("\r\n")
    return resp

def console():
    try:
        while 1:
            cmd = raw_input("> ")
            if cmd:
                s.write(cmd + "\n")
                s.flush()
            print s.readline().rstrip("\r\n")
    except EOFError:
        print "Session closed."

def upload(filename, sector):
    data = open(filename, "rb").read()

    if len(data) != 0x4000:
        print >>sys.stderr, "File", filename, "is not", 0x4000, "bytes long."
        return False

    print >>sys.stderr, "Erasing sector", sector, "..."

    writeToPort("S%x\n" % sector)
    resp = readFromPort()
    if resp != "S":
        print >>sys.stderr, "Sector %d erase failed: %s" % (sector, resp)
        return False

    print >>sys.stderr, "Uploading", filename, "to sector", sector, "..."

    i = 0
    while i < 0x4000:
        value = ord(data[i])
        addr = (sector << 14) | i

        writeToPort("P%04x%02x\n" % (addr, value))
        resp = readFromPort()
        if resp != "P":
            print >>sys.stderr, "Program of location %04x failed: %s" % (addr, resp)
            return False

        i += 1

    return True

if __name__ == "__main__":
    openPort()

    try:
        if "-i" in sys.argv:
            console()
        elif len(sys.argv) > 1:
            if len(sys.argv) > 3 and sys.argv[1] == "-s":
                base = int(sys.argv[2])
                i = 3
            else:
                base = 0
                i = 1
            for sector, filename in enumerate(sys.argv[i:]):
                upload(filename, sector + base)
                sleep(5) # give the device a rest
        else:
            print >>sys.stderr, sys.argv[0], "-i | <filename>..."
            sys.exit(1)
    finally:
        closePort()

# vim: tabstop=4 expandtab shiftwidth=4
